unit alarm_utils;

interface

uses Windows, global_vars, sysUtils, ComCtrls, Forms, Classes, Dialogs, Controls,
iniFiles, dateUtils, Messages;

type
   tAlarmUtil = class
   private
      fListView: tListView;
      count: smallInt;
      appStarting: boolean;
   published
      constructor create;
      destructor free;

      procedure add(const d: tDateTime = 0);

      procedure del(const index: smallInt);
      procedure onlyDel(const index: smallInt);
      procedure selectAfterDel(const index: smallInt);

      procedure delAll(const onlyVisible: boolean = false);
      procedure setAllActive(const value: boolean);
      procedure edit(const index: smallInt);
      procedure listCreate;
      procedure listSelect(const index: smallInt; const selected: boolean);

      procedure setActive(const index: smallInt; const value: boolean);
      procedure editItem(const alarmIndex, itemIndex: smallInt);
      function atsFormShow(const edit: boolean; const d: tDateTime): boolean;
      procedure assignValues(var index: alarms_rec);
      procedure copyValues(const source: alarms_rec; var dest: alarms_rec);

      procedure saveList;
      procedure loadList;

      procedure timeTest;

      procedure timeOut(const index: smallInt);

      procedure showInfo(const index: smallInt);

      function areAnySelected: smallInt;
      procedure selectDate(const value: tDate);

      property listView: tListView read fListView write fListView;

      procedure moveToGroup(sender: tObject);
      function getFirstSelected: smallInt;
      //
      procedure ctrlTimerOnTimer(s: tObject);
      procedure addGroup(sender: tObject);
      procedure setAlarmIndexes;

      procedure setValues(var index: alarms_rec; const d: tDateTime);
   end;
var
   alarm: tAlarmUtil;
implementation

uses ats, utils, soundPlay, calendarEdit, groupEdit,
  Menus, popUp;

{ tAlarm }

procedure{   }tAlarmUtil.add(const d: tDateTime);
var
c: smallInt;
begin
   if not atsFormShow(false, d) then exit;
   count := high(alarms);
   setLength(alarms, count+2);
   inc(Count);
   assignValues(alarms[count]);
   if areAnySelected <= 1 then
   begin
      alarms[count].selected := true;
      for c := 0 to count - 1 do
      alarms[c].selected := false;
   end;
   listCreate;
end;

function{ ,  }tAlarmUtil.atsFormShow(const edit: boolean; const d: tDateTime): boolean;
var
allRight, error: boolean;
mr, counter: smallInt;
messageText: string;
begin
   allRight := false;
   with at_addEdit_form do
   begin
      if isWindowVisible(handle) then
      begin
         messageDlg(' /  !'+#13#10+
         '      !', mtWarning, [mbOk], 0);
         exit;
      end;
      
      clientHeight := 412;
      if high(groups) < 0 then
      begin
         groupSelect.Enabled := false;
         label13.enabled := false;
      end
      else
      begin
         groupSelect.Enabled := true;
         label13.enabled := true;
      end;

      if not edit then
      begin
         editing := false;
         caption := ' ';
         icon.LoadFromResourceName(hInstance, 'add');
         if utils_form.s1_btn.checked and le_alarm.exists then
            setValues(le_alarm.rec, d)
         else
         begin
            timeEdit.Time := time;

            if d = 0 then
               dateEdit.Date := date
            else
               dateEdit.Date := d;

            if wav_file_input.Items.Count > 0 then
              //wav_file_input.Text := wav_file_input.Items.Strings[0]
              wav_file_input.ItemIndex := 0
              else wav_file_input.Text := '';
            note_input.Clear;
            ring_nums_edit.Value := 1;
            active_button.checked := true;
            popUp_button.Checked := true;
            rings_unlimit_button.checked := false;
            ring_nums_edit.Enabled := true;
            reset_onesInDay_button.Enabled := false;
            a_del_button.checked := false;
            a_active_button.checked := false;
            a_mode.ItemIndex := 0;
            groupSelect.ItemIndex := -1;
            for counter := 1 to 7 do
               a_days.Checked[counter-1] := true;
            dateEdit.Enabled := true;
            timeEdit.Enabled := true;
            a_days.Enabled := false;

            onAppRun.Checked := false;
            onesInDay.Checked := false;
            onesInDay.Enabled := false;

            useSpace.Checked := false;
            useSpace.Enabled := false;
            label10.Enabled := false;
            label11.Enabled := false;
            activeFrom.Enabled := false;
            activeTo.Enabled := false;

            activeFrom.value := hourOf(time);
            activeTo.value := hourOf(time);

            use_sound_button.Checked := true;
            use_wav_button.Checked := true;
            use_sound_button.OnClick(nil);
         end;

         onesInDay.OnClick(nil);
      end
      else
      begin
         editing := true;
         caption := ' ';
         icon.LoadFromResourceName(hInstance, 'edit');

         setValues(alarms[al_select_index], 0);

         timeEdit.Enabled := not alarms[al_select_index].onAppRun;

         a_days.Enabled := a_mode.ItemIndex = 1;
         dateEdit.Enabled := (a_mode.ItemIndex <> 1);

         onAppRun.OnClick(nil);
         use_sound_button.OnClick(nil);
      end;
      alarm_panel.visible := true;
      timer_panel.visible := false;
   end;
   repeat
      isOk := false;
      messageText := '';
      error := false;
      with at_addEdit_form do
      begin
         showModal;
         if not isOk then exit;

         if use_wav_button.checked and use_sound_button.Checked and not fileExists(wav_file_input.Text) then
         begin
            error := true;
            messageText := messageText+'    !'+#13#10;
         end;

         if onAppRun.Checked and useSpace.Checked and
           (activeTo.Value < activeFrom.Value) then
         begin
            error := true;
            messageText := messageText+'     !'+#13#10;
         end;

      end;
      if error then
      begin
         mr := messageDlg(messageText+'?', mtWarning, mbYesNoCancel, 0);
         if mr = mrYes then
            allRight := true;
         if mr = mrNo then
            allRight := false;
         if mr = mrCancel then
         begin
            result := false;
            exit;
         end;
      end else allRight := true;
   until allRight;
   result := isOk;
end;

procedure tAlarmUtil.copyValues(const source: alarms_rec; var dest: alarms_rec);
var
counter: shortInt;
begin
    dest.useSpace := source.useSpace;
    dest.activeFrom := source.activeFrom;
    dest.activeTo := source.activeTo;
    dest.showPopUp := source.showPopUp;
    dest.del_alarm := source.del_alarm;
    dest.set_unactive := source.set_unactive;
    dest.rings_unlimit := source.rings_unlimit;
    dest.onAppRun := source.onAppRun;
    dest.onesInDay := source.onesInDay;
    dest.set_on := source.set_on;
    //dest.set_in := date+time;
    dest.groupIndex := source.groupIndex; //groups
    dest.note := source.note;
    dest.music_way := source.music_way;
    dest.active := source.active;
    dest.use_wav := source.use_wav;
    dest.say_time := source.say_time;
    dest.rings_num := source.rings_num;
    for counter := 1 to 7 do
       dest.days[counter] := source.days[counter];
    dest.mode := source.mode;
end;

constructor{  }tAlarmUtil.create;
begin
   fListView := tListView.create(application);
   appStarting := true;
end;

procedure tAlarmUtil.ctrlTimerOnTimer(s: tObject);
begin
   ctrlPress := getAsyncKeyState(162) <> 0;
end;

procedure{   }tAlarmUtil.del(const index: smallInt);
var
c, itemIndex: smallInt;
begin
   {itemIndex := alarms[index].itemIndex;

   count := high(alarms);

   for c := index to count - 1 do
   alarms[c] := alarms[c + 1];

   setLength(alarms, count);

   listCreate;
   //count := high(alarms);
   count := fListView.Items.Count - 1;

   if count >= 0 then
      if itemIndex > count then
         al_select_index := count
      else
         al_select_index := itemIndex
   else
      al_select_index := -1;

   {showMessage('Count: '+intTOStr(count)+'; alIndex: '+intToStr(al_select_index)+
   '; itemIndex: '+intToStr(itemIndex));}

   if al_select_index >= 0 then
      flistView.Items[al_select_index].Selected := true
   else
      listSelect(-1, false);

   {if al_select_index >= 0 then
     flistView.Items[al_select_index].Selected := true;}

   // 2   , ..  listCreate;
   {for c := 0 to flistView.Items.Count - 1 do
     flistView.Items[c].Selected := alarms[alarmIndexes[c]].selected;}

   //utils_form.del_all_alarms_button.enabled := count >= 0;

   {try
      flistView.SetFocus;
   except

   end;}
   onlyDel(index);
   selectAfterDel(index);
end;

procedure{    }tAlarmUtil.delAll(const onlyVisible: boolean = false);
var
c: smallInt;
haveToDel: array of smallInt;
begin
   if not onlyVisible then
   begin
      setLength(alarms, 0);
      fListView.Items.Clear;
      listSelect(-1, false);
      {utils_form.del_all_alarms_button.enabled := false;
      utils_form.a_del_all.Enabled := false;
      utils_form.a_allActive.Enabled := false;
      utils_form.a_allUnActive.Enabled := false;}
      listCreate;
   end
   else
   begin
      if high(groups) >= 0 then
      begin
         for c := 0 to high(alarms) do
         begin
            if alarms[c].groupIndex >= 0 then
            begin
               if groups[alarms[c].groupIndex].visible then
               begin
                  setLength(haveToDel, high(haveToDel)+2);
                  haveToDel[high(haveToDel)] := c;
               end;
            end
            else
            begin
               setLength(haveToDel, high(haveToDel)+2);
               haveToDel[high(haveToDel)] := c;
            end;
         end;

         for c := high(haveToDel) downto 0 do
            onlyDel(haveToDel[c]);

         listCreate;
         listSelect(-1, false);
         
         //showMessage(intToStr(haveToDel[high(haveToDel)]));
         //selectAfterDel(haveToDel[high(haveToDel)]);
      end
      else
      begin
         setLength(alarms, 0);
         fListView.Items.Clear;
         listSelect(-1, false);
         listCreate;
      end;
   end;
end;

procedure{  }tAlarmUtil.edit(const index: smallInt);
begin
   alarms[index].editing := true;
   editItem(index, alarms[index].itemIndex);
   al_select_index := index;
   if not atsFormShow(true, 0) then
   begin
      alarms[index].editing := false;
      editItem(index, alarms[index].itemIndex);
      exit;
   end;
   alarms[index].editing := false;
   assignValues(alarms[index]);
   editItem(index, alarms[index].itemIndex);
   listSelect(alarms[index].itemIndex, index >= 0);
   //listCreate;
   calendarRepaint;

   {for c := 0 to flistView.Items.Count - 1 do
     flistView.Items[c].Selected := alarms[alarmIndexes[c]].selected;}
end;

procedure{   }tAlarmUtil.editItem(const alarmIndex, itemIndex: smallInt);
var
tsl: tStringList;
ts: string;
setOn: string;
begin
   tsl := tStringList.Create;
   ts := '';
   //fListView.Items.BeginUpdate;
   with fListView.Items.Item[itemIndex] do
   begin
      if alarms[alarmIndex].onAppRun then
      begin
        setOn := ' ';
        if alarms[alarmIndex].useSpace then
          if alarms[alarmIndex].activeFrom = alarms[alarmIndex].activeTo then
            setOn := setOn + ' ('+intToStr(alarms[alarmIndex].activeFrom)+'.)'
          else
            setOn := setOn + ' ('+intToStr(alarms[alarmIndex].activeFrom)+
            '.-'+intToStr(alarms[alarmIndex].activeTo)+'.)';

      end
      else
        setOn := timeToStr(alarms[alarmIndex].set_on);

      case alarms[alarmIndex].mode of
      0:
      begin
         ts := ' ';
         setOn := setOn+' ['+DateToStr(alarms[alarmIndex].set_on)+']';
      end;
      1: ts := '';
      2:
      begin
         ts := '';
         setOn := setOn + ' ['+intToStr(dayOf(alarms[alarmIndex].set_on))+'- ]';
      end;
      3:
      begin
         ts := '';
         setOn := setOn + ' ['+intToStr(dayOf(alarms[alarmIndex].set_on))+'- '+
            months[monthOf(alarms[alarmIndex].set_on)]+']';
      end;
      end;
      caption := intToStr(index+1);
      tsl.Clear;
      tsl.Add(setOn);
      tsl.Add(alarms[alarmIndex].note);

      if high(groups) < alarms[alarmIndex].groupIndex then
         alarms[alarmIndex].groupIndex := -1;

      if (alarms[alarmIndex].groupIndex >= 0) and (high(groups) >= 0) then
         tsl.Add(groups[alarms[alarmIndex].groupIndex].name)
      else
         tsl.Add(' ');
      {if not "set_in"
      tsl.Add(timeToStr(alarms[index].set_in)+' ['+
         DateToStr(alarms[index].set_in)+']');}

      tsl.Add(ts);

      if not alarms[alarmIndex].editing then
      begin
         if alarms[alarmIndex].active then
         tsl.Add('')
         else
         tsl.Add('');
      end
      else tsl.Add('');

      if alarms[alarmIndex].use_wav then
      begin
         if alarms[alarmIndex].rings_unlimit then
         ts := '  "'+alarms[alarmIndex].music_way+'"; '
         else
         ts := ' "'+alarms[alarmIndex].music_way+'" '+intToStr(alarms[alarmIndex].rings_num)+
         ' .; ';
      end else ts := '';
      if alarms[alarmIndex].say_time then
        ts := ' ; ';

      if alarms[alarmIndex].showPopUp then
         ts := ts + '  ; ';
      
      if alarms[alarmIndex].del_alarm then
      ts := ts + ' ; ';
      if alarms[alarmIndex].set_unactive then
      ts := ts + '  ; ';
      tsl.Add(ts);
      subItems := tsl;

      alarms[alarmIndex].itemIndex := itemIndex;
      alarmIndexes[itemIndex] := alarmIndex;
   end;
   tsl.free;
   //fListView.Items.EndUpdate;
end;

destructor{   }tAlarmUtil.free;
begin
   listView.Free;
end;

function{   }tAlarmUtil.getFirstSelected: smallInt;
var
c: smallInt;
begin
   result := -1;
   if high(alarms) < 0 then exit;

   for c := 0 to high(alarms) do
   begin
      if alarms[c].selected then
      begin
         result := c;
         exit;
      end;
   end;
end;

procedure{   }tAlarmUtil.listCreate;
var
c: smallInt;
lineIndex: smallInt;
procedure edit;
begin
   fListView.Items.Add;
   setLength(alarmIndexes, high(alarmIndexes)+2);
   inc(lineIndex);
   alarmIndexes[lineIndex] := c;

   editItem(c, lineIndex);
end;
procedure setEnable(const value: boolean);
begin
   utils_form.del_all_alarms_button.enabled := value;
   utils_form._alarmDelAll.Enabled := value;

   utils_form.a_allActive.Enabled := value;
   utils_form.a_allUnActive.Enabled := value;

   utils_form.N38.Enabled := value;
   utils_form.N41.Enabled := value;
   utils_form.N1.Enabled := value;

   utils_form.a_selectedActive.Enabled := value;
   utils_form.a_selectedUnActive.Enabled := value;

   utils_form.a_selectActive.Enabled := value;
   utils_form.a_selectUnActive.Enabled := value;

   utils_form.a_selectAll.Enabled := value;
   utils_form.a_selectToday.Enabled := value;
   utils_form.a_selectInvert.Enabled := value;
end;
begin
   //
   {utils_form.a_edit.Enabled := false;
   utils_form.a_del.Enabled := false;
   utils_form.a_active.Enabled := false;
   utils_form.alarm_edit_button.Enabled := false;
   utils_form.alarm_del_button.Enabled := false;
   utils_form.alarm_active_button.Enabled := false;}
   //

   fListView.Items.Clear;
   {}setLength(alarmIndexes, 0);
   calendarRepaint;
   count := high(alarms);

   if count < 0 then
   begin
      setEnable(false);

      if high(groups) >= 0 then
         for c := 0 to high(groups) do
            groups[c].alarmsCount := 0;

      groupEdit_form.listCreate;

      exit;
   end;

   lineIndex := -1;

   fListView.Items.BeginUpdate;

   if high(groups) >= 0 then
      for c := 0 to high(groups) do
         groups[c].alarmsCount := 0;

   for c := 0 to count do
   begin
      if (alarms[c].groupIndex > -1) then
      begin
         inc(groups[alarms[c].groupIndex].alarmsCount);

         if groups[alarms[c].groupIndex].visible then
            edit
         else
         alarms[c].selected := false;
      end
      else
         if (alarms[c].groupIndex = -1) then
            edit
         else
            alarms[c].selected := false;
      end;

   {for c := 0 to high(alarms) do
     flistView.Items[alarms[c].itemIndex].Selected := alarms[c].selected;}

   for c := 0 to flistView.Items.Count - 1 do
     flistView.Items[c].Selected := alarms[alarmIndexes[c]].selected;

   fListView.Items.EndUpdate;

   setEnable(fListView.Items.Count > 0);

   calendarRepaint;
   groupEdit_form.listCreate;
end;

procedure{   }tAlarmUtil.listSelect(const index: smallInt; const selected: boolean);
var
anySelected: boolean;
selectedCount: smallInt;
begin
   if selected then al_select_index := alarmIndexes[index] else al_select_index := -1;

   selectedCount := 0;

   if index >= 0 then
   begin
      alarms[alarmIndexes[index]].selected := selected;
      selectedCount := areAnySelected;
      anySelected := selectedCount > 0;
   end else
      anySelected := false;

   al_first_selected := getFirstSelected;

   if anySelected then   
      statusBarCaption := ': '+
      intToStr(selectedCount)+'/'+intToStr(listView.Items.Count)
   else
      statusBarCaption := '';

   with utils_form do
   begin
      alarm_edit_button.enabled := anySelected;
      alarm_del_button.enabled := anySelected;
      alarm_active_button.enabled := anySelected;

      _alarmEdit.Enabled := anySelected;
      _alarmdel.Enabled := anySelected;
      _alarmactive.Enabled := anySelected;
      _alarmshowInfo.Enabled := anySelected;

      a_selectedActive.enabled := anySelected;
      a_selectedUnActive.enabled := anySelected;

      _moveIntoGroup.Enabled := anySelected and (_moveIntoGroup.Count > 0);
      
      if al_first_selected >= 0 then
      begin
        alarm_active_button.down:= alarms[al_first_selected].active;
        _alarmactive.Checked := alarms[al_first_selected].active;
      end;
      status_bar.Panels[0].Text := statusBarCaption;
   end;
end;

procedure{   }tAlarmUtil.loadList;
var
f: tIniFile;
ts, f_way: string;
c1, c2, mh, ah, loadMode, changeStatus: smallInt;
active: boolean;
begin
   f_way := extractFilePath(application.ExeName)+'_work\alarms.ini';
   //if (not fileExists(f_way)) or (not utils_form.s11_btn.checked) then exit;
   f := tIniFile.Create(f_way);

   //***   ***
   mh := f.ReadInteger('Count', 'Groups', -1);
   if mh >= 0 then
   begin
      for c1 := 0 to mh do
      begin
         ts := 'Group'+intToStr(c1);
         setLength(groups, high(groups)+2);
         groups[c1].name := f.ReadString(ts, 'name', '( )');
         groups[c1].visible := f.ReadBool(ts, 'visible', false);
      end;
      groupEdit_form.listCreate;
   end;
   //*** ***************** ***
   mh := f.ReadInteger('Count', 'Count', -1);
   //le_alarm := f.ReadInteger('Count', 'LastEdited', -1);

   if (mh = -1) or (not utils_form.s11_btn.checked) then
   begin
      f.Free;
      exit;
   end;
   //setLength(alarms, mh+1);

   loadMode := utils_form.s12_edit.ItemIndex;
   changeStatus := utils_form.s13_edit.ItemIndex;

   ah := -1;

   for c1 := 0 to mh do
   begin
      ts := 'Alarm'+intToStr(c1);

      active := f.ReadBool(ts, 'active', false);

      if (active and (loadMode = 2)) or (not active and (loadMode = 1)) then
        continue;

      if utils_form.s32_btn.Checked then
      begin
         if (f.ReadInteger(ts, 'mode', 0) = 0) then
         begin
            if f.ReadBool(ts, 'onAppRun', false) then
            begin
               {if f.ReadBool(ts, 'onesInDay', false) then
                  if dateOf(f.ReadFloat(ts, 'rang_in', 0)) > 0 then
                     continue;}
               if dateOf(f.ReadFloat(ts, 'set_on', 0)) < date then continue;
               //if dateOf(f.ReadFloat(ts, 'rang_in', 0)) < date then continue;
            end
            else
               if f.ReadFloat(ts, 'set_on', date+time) < time + date then
                  continue;
         end;

            {(f.ReadFloat(ts, 'set_on', date+time) < time + date) then
               continue;}
      end;

      case changeStatus of
      1: if active then active := false;
      2: if not active then active := true;
      3: active := not active;
      end;
      //active := active <> changeStatus;

      inc(ah);
      setLength(alarms, ah+1);

      alarms[ah].active := active;
      alarms[ah].editing := false;
      alarms[ah].showPopUp := f.ReadBool(ts, 'popUp', true);
      alarms[ah].set_unactive := f.ReadBool(ts, 'set_unactive', false);
      alarms[ah].onAppRun := f.ReadBool(ts, 'onAppRun', false);
      alarms[ah].onesInDay := f.ReadBool(ts, 'onesInDay', false);
      alarms[ah].rings_unlimit := f.ReadBool(ts, 'rings_unlimit', false);
      alarms[ah].rings_num := f.ReadInteger(ts, 'rings_num', 1);
      alarms[ah].del_alarm := f.ReadBool(ts, 'del_alarm', true);
      alarms[ah].note:= f.ReadString(ts, 'note', '');
      alarms[ah].music_way := f.ReadString(ts, 'music_way', at_addedit_form.wav_file_input.Text);
      alarms[ah].use_wav := f.ReadBool(ts, 'use_wav', false);
      alarms[ah].say_time := f.ReadBool(ts, 'say_time', false);
      for c2 := 1 to 7 do
         alarms[ah].days[c2] := f.ReadBool(ts, 'day'+intToStr(c2), true);
      alarms[ah].mode := f.ReadInteger(ts, 'mode', 0);
      alarms[ah].set_on := f.ReadFloat(ts, 'set_on', date+time);
      //alarms[ah].set_in := f.ReadFloat(ts, 'set_in', date+time);
      alarms[ah].rang_in := f.ReadFloat(ts, 'rang_in', 0);
      alarms[ah].activeFrom := f.ReadInteger(ts, 'activeFrom', 0);
      alarms[ah].activeTo := f.ReadInteger(ts, 'activeTo', 0);
      alarms[ah].groupIndex := f.ReadInteger(ts, 'groupIndex', 0);
      alarms[ah].useSpace := f.ReadBool(ts, 'useSpace', true);
      alarms[ah].selected := false;
   end;

   f.Free;

   le_alarm.exists := false;
end;

procedure{    }tAlarmUtil.moveToGroup(sender: tObject);
var
c: smallInt;
begin
   if high(alarms) < 0  then exit;
   for c := 0 to high(alarms) do
      if alarms[c].selected then
      begin
         alarms[c].groupIndex := (sender as tMenuItem).Tag;
         if alarms[c].groupIndex >= 0 then
            fListView.Items[alarms[c].itemIndex].Selected :=
            groups[alarms[c].groupIndex].visible;
      end;
   listCreate;
end;

procedure{   }tAlarmUtil.onlyDel(const index: smallInt);
var
c: smallInt;
begin
   //ac_item_index := alarms[index].itemIndex;

   count := high(alarms);

   for c := index to count - 1 do
   alarms[c] := alarms[c + 1];

   setLength(alarms, count);
end;

procedure{   }tAlarmUtil.saveList;
var
f: tIniFile;
f_way, ts: string;
c1, c2, mh: smallInt;
begin
   try
   f_way := extractFilePath(application.ExeName)+'_work\alarms.ini';

   if fileExists(f_way) then deleteFile(f_way);
   f := tIniFile.Create(f_way);

   mh := high(groups);
   f.WriteInteger('Count', 'Groups', mh);

   if mh >= 0 then
      for c1 := 0 to mh do
      begin
         ts := 'Group'+intToStr(c1);
         f.WriteString(ts, 'name', groups[c1].name);
         f.WriteBool(ts, 'visible', groups[c1].visible);
      end;

   mh := high(alarms);
   f.WriteInteger('Count', 'Count', mh);
   //f.WriteInteger('Count', 'LastEdited', le_alarm);

   if mh >= 0 then
      for c1 := 0 to mh do
      begin
         ts := 'Alarm'+intToStr(c1);
         f.writeBool(ts, 'active', alarms[c1].active);
         f.WriteBool(ts, 'popUp', alarms[c1].showPopUp);
         f.WriteBool(ts, 'onAppRun', alarms[c1].onAppRun);
         f.WriteBool(ts, 'onesInDay', alarms[c1].onesInDay);
         f.writeBool(ts, 'set_unactive', alarms[c1].set_unactive);
         f.writeBool(ts, 'rings_unlimit', alarms[c1].rings_unlimit);
         f.writeInteger(ts, 'rings_num', alarms[c1].rings_num);
         f.writeBool(ts, 'del_alarm', alarms[c1].del_alarm);
         f.WriteString(ts, 'note', alarms[c1].note);
         f.WriteString(ts, 'music_way', alarms[c1].music_way);
         f.WriteBool(ts, 'use_wav', alarms[c1].use_wav);
         f.WriteBool(ts, 'say_time', alarms[c1].say_time);
         for c2 := 1 to 7 do
            f.WriteBool(ts, 'day'+intToStr(c2), alarms[c1].days[c2]);
         f.writeInteger(ts, 'mode', alarms[c1].mode);
         f.WriteFloat(ts, 'set_on', alarms[c1].set_on);
         //f.WriteFloat(ts, 'set_in', alarms[c1].set_in);
         f.WriteFloat(ts, 'rang_in', alarms[c1].rang_in);
         f.WriteInteger(ts, 'activeFrom', alarms[c1].activeFrom);
         f.Writeinteger(ts, 'activeTo', alarms[c1].activeTo);
         f.Writeinteger(ts, 'groupIndex', alarms[c1].groupIndex);
         f.writeBool(ts, 'useSpace', alarms[c1].useSpace);
      end;
   f.Free;
   except

   end;
end;

procedure tAlarmUtil.setAlarmIndexes;
var
c: smallInt;
begin
   if fListView.Items.Count = 0 then exit;
   setLength(alarmIndexes, fListView.Items.Count);

   for c := 0 to high(alarms) do
      alarmIndexes[alarms[c].itemIndex] := c;
end;

procedure{    / }tAlarmUtil.setAllActive(const value: boolean);
var
counter: smallInt;
begin
   count := high(alarms);
   if count < 0 then exit;
   counter := 0;
   while counter <= count do
   begin
      if alarms[counter].groupIndex >= 0 then
      begin
         if groups[alarms[counter].groupIndex].visible then
            alarms[counter].active := value;
      end
      else
         alarms[counter].active := value;
         
      inc(counter);
   end;
   listCreate;
   //if al_select_index >= 0 then flistView.Items[al_select_index].Selected := true;
end;


procedure tAlarmUtil.setValues(var index: alarms_rec; const d: tDateTime);
var
counter: smallInt;
begin
   with at_addedit_form do
   begin
      timeEdit.time := index.set_on;

      if d = 0 then
         dateEdit.date := index.set_on
      else
         dateEdit.date := d;

      use_sound_button.checked := index.use_wav or
         index.say_time;
      use_wav_button.Checked := index.use_wav;
      say_time_button.Checked := index.say_time;

      rings_unlimit_buttonClick(nil);
      wav_file_input.Text := index.music_way;
      note_input.Text := index.note;
      ring_nums_edit.Value := index.rings_num;
      active_button.checked := index.active;
      popUp_button.checked := index.showPopUp;
      rings_unlimit_button.checked := index.rings_unlimit;
      a_del_button.checked := index.del_alarm;
      a_active_button.checked := index.set_unactive;
      a_mode.ItemIndex := index.mode;
      groupSelect.ItemIndex := index.groupIndex;
      for counter := 1 to 7 do
         a_days.Checked[counter-1] := index.days[counter];

      onAppRun.Checked := index.onAppRun;
      onesInDay.Checked := index.onesInDay;
      onesInDay.Enabled := index.onAppRun;
         //reset_onesInDay_button.Enabled := not utils_form.s27_btn.Checked;

      useSpace.Checked := index.useSpace;
      useSpace.Enabled := index.onAppRun;
      label10.Enabled := index.useSpace
         and index.onAppRun;
      label11.Enabled := index.useSpace
         and index.onAppRun;
      activeFrom.Enabled := index.useSpace
         and index.onAppRun;
      activeTo.Enabled := index.useSpace
         and index.onAppRun;

      activeFrom.value := index.activeFrom;
      activeTo.value := index.activeTo;
   end;
end;

procedure{   }tAlarmUtil.selectAfterDel(const index: smallInt);
begin
   listCreate;

   count := fListView.Items.Count - 1;

   if count >= 0 then
      if index > count then
         al_select_index := count
      else
         al_select_index := index
   else
      al_select_index := -1;

   if al_select_index >= 0 then
      flistView.Items[al_select_index].Selected := true
   else
      listSelect(-1, false);

   try
      flistView.SetFocus;
   except

   end;
end;

procedure{     .. }tAlarmUtil.selectDate(const value: tDate);
var
c: smallInt;
//multiNotPress: Boolean;
mustSelect: boolean;
begin
   {count := high(alarms);
   if count < 0 then exit;

   multiNotPress := getasynckeystate(mSelectKey) = 0;

   for counter := 0 to count do
   begin
      mustSelect := false;
      case alarms[counter].mode of
      0: mustSelect := (dateOf(alarms[counter].set_on) = value) and calendarEdit_form.calendar_mode.Checked[0];
      1: mustSelect := alarms[counter].days[dayOfWeek(value)] and calendarEdit_form.calendar_mode.Checked[1];
      2: mustSelect := (dayOf(alarms[counter].set_on) = dayOf(value)) and calendarEdit_form.calendar_mode.Checked[2];
      3: mustSelect := (dayOf(alarms[counter].set_on) = dayOf(value)) and
         (monthOf(alarms[counter].set_on) = monthOf(value)) and calendarEdit_form.calendar_mode.Checked[3];
      end;

      if mustSelect then
      begin
         if not alarms[counter].active and
            calendarEdit_form.calendar_mode.Checked[4] then
         begin
            if multiNotPress then alarm.listView.Items[alarms[counter].itemIndex].Selected := false;
            continue;
         end;
         alarm.listView.Items[alarms[counter].itemIndex].Selected := true;
      end
      else
         if multiNotPress then alarm.listView.Items[alarms[counter].itemIndex].Selected := false;}


   count := alarm.listView.Items.Count;
   if count = 0 then exit;

   //multiNotPress := getasynckeystate(mSelectKey) = 0;

   for c := 0 to Count - 1 do
   begin
      mustSelect := false;
      case alarms[alarmIndexes[c]].mode of
      0: mustSelect := (dateOf(alarms[alarmIndexes[c]].set_on) = value) and calendarEdit_form.calendar_mode.Checked[0];
      1: mustSelect := alarms[alarmIndexes[c]].days[dayOfWeek(value)] and calendarEdit_form.calendar_mode.Checked[1];
      2: mustSelect := (dayOf(alarms[alarmIndexes[c]].set_on) = dayOf(value)) and calendarEdit_form.calendar_mode.Checked[2];
      3: mustSelect := (dayOf(alarms[alarmIndexes[c]].set_on) = dayOf(value)) and
      (monthOf(alarms[alarmIndexes[c]].set_on) = monthOf(value)) and calendarEdit_form.calendar_mode.Checked[3];
      end;

      if mustSelect then
      begin
         if (not alarms[alarmIndexes[c]].active and
            calendarEdit_form.calendar_mode.Checked[4]) then
         begin
            if not ctrlPress then alarm.listView.Items[c].Selected := false;
            continue;
         end;

         {if (alarms[c].groupIndex >= 0) and (high(groups) >= 0) then
         begin
            if not groups[alarms[c].groupIndex].visible and
               calendarEdit_form.calendar_mode.Checked[5] then
                  if multiNotPress then alarm.listView.Items[c].Selected := false;
                     continue;
         end;}
         
         alarm.listView.Items[c].Selected := true;
      end
      else
         if not ctrlPress then alarm.listView.Items[c].Selected := false;
   end;

      {if multiNotPress then
      begin
         if mustSelect then
         begin
            if not alarms[counter].active and utils_form.s28_btn.Checked then
            begin
               alarm.listView.Items[counter].Selected := false;
               continue;
            end;
            alarm.listView.Items[counter].Selected := true;
         end
         else
            alarm.listView.Items[counter].Selected := false;
      end
      else
      begin
         if mustSelect then
         begin
            if not alarms[counter].active and utils_form.s28_btn.Checked then
            begin
               alarm.listView.Items[counter].Selected := false;
               continue;
            end;
            alarm.listView.Items[counter].Selected := not alarms[counter].selected;
         end;
      end;

   end;}
   try
      alarm.listView.SetFocus;
   except

   end;
end;

procedure{   }tAlarmUtil.setActive(const index: smallInt; const value: boolean);
begin
   alarms[index].active := value;
   editItem(index, alarms[index].itemIndex);
   {}utils_form._alarmactive.Checked := value;
   {}utils_form.alarm_active_button.Down := value;
   calendarRepaint;
end;

procedure{   }tAlarmUtil.showInfo(const index: smallInt);
var
str: string;
counter: smallInt;
begin
   str := '  '+alarm.listView.Items[alarms[index].itemIndex].Caption+';';
   str := str+#13#10+' : '+alarm.listView.Items[alarms[index].itemIndex].SubItems[0]+';';
   {if alarms[index].mode <> 1 then
      str := str+' ['+DateToStr(alarms[index].set_on)+'];' else
      str := str +';';}
   str := str + #13#10+': ';
   case alarms[index].mode of
   0: str := str + ' ;';
   1: str := str + ';';
   2: str := str + ';';
   3: str := str + ';';
   end;
   if alarms[index].mode = 1 then
   begin
      str := str + #13#10;
      for counter := 1 to 7 do
         if alarms[index].days[counter] then
            str := str + days[counter]+'; ';
   end;
   str := str + #13#10;
   str := str + ': "'+alarms[index].note+'";';
   str := str + #13#10;
   if (alarms[index].groupIndex >= 0) and (high(groups) >= 0) then
      str := str + ': '+groups[alarms[index].groupIndex].name
   else
      str := str + ' ';

   str := str + #13#10+': ';
   if alarms[index].active then
     str := str + ';' else str := str + ';';
   str := str + #13#10+': ';
   if alarms[index].use_wav then
   begin
     if alarms[index].rings_unlimit then
     str := str + '  "'+alarms[index].music_way+'"; '
     else
     str := str + ' "'+alarms[index].music_way+'" '+intToStr(alarms[index].rings_num)+
     ' .; ';
   end;

   if alarms[index].say_time then
     str := str + ' ; ';

   if alarms[index].showPopUp then
     str := str + #13#10+'  ; ';

   str := str + #13#10;
   if alarms[index].del_alarm then
   str := str + ' ;';
   if alarms[index].set_unactive then
   str := str + '  ;';

   str := str + #13#10;
   if alarms[index].rang_in <> 0 then
   str := str + '   : '+dateTimeToStr(alarms[index].rang_in)+';'
   else
   str := str + '   ;';

   messageDlg(str, mtInformation, [mbOk], 0);
end;


procedure{   }tAlarmUtil.timeOut(const index: smallInt);
var
caption: string;
begin
   setRinging(true);

   caption := timeToStr(time) + ' ['+DateToStr(date)+'] : '+alarms[index].note;

   alarms[index].rang_in := time+date;

   if alarms[index].showPopUp then
   begin
      nowRingingPage := 0;
      showPopUpWnd(' !!!', caption);
   end;

   if noteHistory = '' then
      noteHistory := caption
   else
      noteHistory := caption+#13#10+noteHistory;

   if alarms[index].use_wav then
   begin
      if fileExists(alarms[index].music_way) then
      begin
         _player.stop;
         _player.way := alarms[index].music_way;
         _player.repeatNums := alarms[index].rings_num;
         _player.RepeatUnlimit := alarms[index].rings_unlimit;
         a_haveToPlay := true;
         //_player.play;
      end;
   end;

   if alarms[index].say_time and (@sayTime <> nil) and AllowPlaying then
   begin
      AllowPlaying(true);
      sayTime(hourOf(time), minuteOf(time), messageSendHandle, WM_USER+1);
   end;

   if alarms[index].set_unactive then alarms[index].active := false;
   //if alarms[index].del_alarm then onlydel(index);// else editItem(index, alarms[index].itemIndex);

   //alarm.listView.Repaint;
end;

procedure{          }tAlarmUtil.addGroup(sender: tObject);
var
c, h: smallInt;
begin
   if high(alarms) < 0  then exit;
   h := high(groups);

   groupEdit_form.add_button.OnClick(nil);
   if h = high(groups) then exit;
   h := high(groups);

   if messageDlg('  "'+groups[h].name+'" ?', mtConfirmation, mbYesNo, 0) = mrYes then
   groups[h].visible := true else groups[h].visible := false;


   for c := 0 to high(alarms) do
      if alarms[c].selected then
      begin
         alarms[c].groupIndex := h;
         if alarms[c].groupIndex >= 0 then
            fListView.Items[alarms[c].itemIndex].Selected :=
            groups[alarms[c].groupIndex].visible;
      end;
   listCreate;
end;

function{   }tAlarmUtil.areAnySelected: smallInt;
var
c: smallInt;
begin
   result := 0;
   for c := 0 to high(alarms) do
      if alarms[c].selected then inc(result);
end;

procedure{   }tAlarmUtil.assignValues(var index: alarms_rec);
var
counter: smallInt;
begin
  with at_addEdit_form do
  begin
    index.selected := false;
    if utils_form.s27_btn.Checked then
      index.rang_in := 0;
    index.editing := false;
    index.useSpace := useSpace.Checked;
    index.activeFrom := activeFrom.value;
    index.activeTo := activeTo.value;
    index.showPopUp := popUp_button.Checked;
    index.del_alarm := a_del_button.checked;
    index.set_unactive := a_active_button.checked;
    index.rings_unlimit := rings_unlimit_button.checked;
    index.onAppRun := onAppRun.Checked;
    index.onesInDay := onesInDay.Checked;
    index.set_on := timeOf(timeEdit.DateTime)+dateOf(dateEdit.DateTime);
    //index.set_in := date+time;
    index.groupIndex := groupSelect.ItemIndex; //groups
    index.note := note_input.Text;
    index.music_way := wav_File_input.Text;
    index.active := active_button.checked;
    index.use_wav := use_wav_button.checked and use_sound_button.Checked;
    index.say_time := say_time_button.checked and use_sound_button.Checked;
    index.rings_num := ring_nums_edit.Value;
    for counter := 1 to 7 do
       index.days[counter] := a_days.Checked[counter-1];
    index.mode := a_mode.ItemIndex;
  end;
  copyValues(index, le_alarm.rec);
  le_alarm.exists := true;
end;

procedure{  }tAlarmUtil.timeTest;
var
counter: smallInt;
del, time_out, dateRight, Limit: boolean;
haveToDel: array of smallInt;
begin
   count := high(alarms);
   if count < 0 then
   begin
      appStarting := false;
      exit;
   end;
   counter := 0;
   a_haveToPlay := false;
   a_haveToRepaint := false;
   //a_wasDeleted := false;
   //fListView.Items.BeginUpdate;
   while counter <= count do
   begin
      if (not alarms[counter].active) or (alarms[counter].editing) then
      begin
         inc(counter);
         continue;
      end;

      time_out := false;
      dateRight := false;

      { ***    *** }
      case alarms[counter].mode of
      0: if (dateToStr(alarms[counter].set_on) = dateToStr(date)) then dateRight := true;
      1: if alarms[counter].days[dayOfWeek(date)] then dateRight := true;
      2: if (dayOf(alarms[counter].set_on) = dayOf(date)) then dateRight := true;
      3: if (monthOf(alarms[counter].set_on) = monthOf(date)) and
            (dayOf(alarms[counter].set_on) = dayOf(date)) then dateRight := true;
      end;
      { ***    *** }

      if not alarms[counter].onAppRun and dateRight and
        (timeToStr(alarms[counter].set_on) = timeToStr(time)) then
           time_out := true;

      if alarms[counter].onAppRun and appStarting and dateRight then
      begin
         if alarms[counter].useSpace then
         begin
            Limit := false;

            if (hourOf(time) <= alarms[counter].activeTo) and
               (hourOf(time) >= alarms[counter].activeFrom) then
            Limit := true;
         end;

         if alarms[counter].onesInDay then
         begin
            if (dateOf(alarms[counter].rang_in) <> date) then
            begin
              if alarms[counter].useSpace then
              begin
                 if Limit then time_out := true;
              end else time_out := true;
            end;
         end
         else
         begin
            if alarms[counter].useSpace then
            begin
              if Limit then time_out := true;
            end else time_out := true;
         end
      end;

      if time_out then
      begin
         if alarms[counter].del_alarm then
         begin
            setLength(haveToDel, high(haveToDel)+2);
            haveToDel[high(haveToDel)] := counter;
         end
         else a_haveToRepaint := true;

         timeOut(counter);
         {if del then
         begin
            if alarms[counter].groupIndex >= 0 then
               if groups[alarms[counter].groupIndex].visible then
               begin
                  editItem(counter, alarms[counter].itemIndex);
                  if al_select_index >= 0 then listSelect(al_select_index, true);
               end;
            inc(counter);
            a_haveToRepaint := true;
         end
         else a_wasDeleted := true;}
         //continue;
      end;
      inc(counter);
   end;
   appStarting := false;

   if high(haveToDel) >= 0 then
   begin
      for counter := high(haveToDel) downto 0 do
         alarm.onlyDel(haveToDel[counter]);

      //if areAnySelected = 0 then
         listCreate
      {else
         alarm.selectAfterDel(lastDeleted);}
   end;

   //if a_wasDeleted then selectAfterDel(-1);
   if a_haveToPlay then _player.play;
   if a_haveToRepaint then fListView.Repaint;
end;

end.
