unit popUpWnd;

interface

uses
  SysUtils, Forms, Classes, Controls, graphics, ComCtrls, popUp;

type
  tHideInterval = 1000..3600000;
  tPopUpPos = (ppBottomRight, ppBottomLeft,
     ppTopRight, ppTopLeft, ppScreenCenter, ppCustom);
  tPopUpWindow = class(tComponent)
  private
    { Private declarations }
    fOnCheck, fOnClose: tNotifyEvent;
    fOnTop: boolean;
    fAutoHide: boolean;
    fHideAfter: tHideInterval;
    fWndLeft, fWndTop, fWndWidth, fWndHeight: integer;
    fPopUpPos: tPopUpPos;
    fBlink: boolean;
    fBlinkInterval: integer;
    fBlinkColor: tColor;
    formVisible: boolean;
    procedure setPos;
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }

     constructor create(aOwner: tComponent); override;
     destructor free;

     procedure show(const c, t: string);
     procedure onCloseClick(Sender: TObject);
     procedure FormClose(Sender: TObject; var Action: TCloseAction);
     procedure close;

     property onCheck: tNotifyEvent read fOnCheck write fOnCheck;
     property onClose: tNotifyEvent read fonClose write fOnClose;

     property onTop: boolean read fOnTop write fOnTop;

     property WndLeft: integer read fWndLeft write fWndLeft;
     property WndTop: integer read fWndTop write fWndTop;
     property WndWidth: integer read fWndWidth write fWndWidth;
     property WndHeight: integer read fWndHeight write fWndHeight;

     property Blink: boolean read fBlink write fBlink;
     property BlinkIntegval: integer read fBlinkInterval write fBlinkInterval;
     property BlinkColor: tColor read fBlinkColor write fBlinkColor;

     property popUpPos: tPopUpPos read fPopUpPos write fPopUpPos;

     property autoHide: boolean read fAutoHide write fAutoHide;
     property hideAfter: tHideInterval read fHideAfter write fHideAfter;
  end;

procedure Register;

implementation

uses
  WinTypes, Windows, Messages;

procedure Register;
begin
   RegisterComponents('atomAltera', [tPopUpWindow]);
end;

{ tTopUpWnd }

procedure tPopUpWindow.close;
begin
   popUp_form.Close;
end;

constructor tPopUpWindow.create(aOwner: tComponent);
begin
   inherited create(aOwner);
   fAutoHide := true;
   fHideAfter := 5000;
   fBlink := true;                          
   fBlinkInterval := 500;
   fBlinkColor := $0000FF;
   popup_form := TpopUp_form.Create(application);
   fOnTop := true;
   fWndLeft := 100;
   fWndTop := 100;
   fWndWidth := 250;
   fWndHeight := 150;
   formVisible := false;
end;

procedure tPopUpWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   popUp_form.autoHideTimer.Enabled := false;
   formVisible := false;
   fWndLeft := popUp_form.Left;
   fWndTop := popUp_form.Top;
   fWndWidth := popUp_form.Width;
   fWndHeight := popUp_form.Height;
   if assigned(fOnClose) then fOnClose(self);
end;

{procedure tPopUpWindow.onFormClick(Sender: TObject);
begin
   close;
end;}

destructor tPopUpWindow.free;
begin
   popup_form.Free;
end;

procedure tPopUpWindow.onCloseClick(Sender: TObject);
begin
   if assigned(fOnCheck) then fOnCheck(self);
   popUp_form.Close;
end;

procedure tPopUpWindow.setPos;
var
rec: tRect;
wnd: hwnd;
tbHeight: word;
begin
   wnd := FindWindow('Shell_TrayWnd', Nil);
   If wnd<>0 then
   begin
      Windows.GetClientRect(wnd, Rec);
      tbHeight := rec.Bottom;
   end else tbHeight := 0;

   case fPopUpPos of
   ppBottomRight:
   begin
      popup_form.left := screen.Width - popup_form.width;
      popup_form.top := screen.height - popup_form.height - tbHeight;
   end;
   ppBottomLeft:
   begin
      popup_form.left := 0;
      popup_form.top := screen.height - popup_form.height - tbHeight;
   end;
   ppTopRight:
   begin
      popup_form.left := screen.Width - popup_form.width;
      popup_form.top := 0;
   end;
   ppTopLeft:
   begin
      popup_form.left := 0;
      popup_form.top := 0;
   end;
   ppScreenCenter:
   begin
      popup_form.left := screen.Width div 2 - popup_form.width div 2;
      popup_form.top := screen.height div 2 - popup_form.height div 2;
   end;
   ppCustom:
   begin
      popup_form.left := fWndLeft;
      popup_form.top := fWndTop;
   end;
   end;
end;

procedure tPopUpWindow.show(const c, t: string);
var
Wnd: HWND;
begin
   with popup_form do
   begin
      autoHideTimer.Enabled := false;

      if formVisible then
        text.text := t+#13#10+text.text
      else
        text.text := t;

      Caption := c;

      {if text.Width > 190 then
         clientWidth := text.Width + 10
         else clientWidth := 200;

      if text.Height > 100 then
         clientHeight := text.height + 35
         else clientHeight := 120;}

      setPos();

      if fOnTop then formStyle := fsStayOnTop else formStyle := fsNormal;

      BlinkTimer.Enabled := fBlink;
      Blinktimer.Interval := fBlinkInterval;
      blinkColor := fBlinkColor;
      //alphaBlend := fAlphaEffect;

      text.color := clBtnFace;
                                                
      text.OnDblClick := onCloseClick;
      OnClose := FormClose;

      width := fWndWidth;
      height := fWndHeight;

      if autoHide then
      begin
         popup_form.autoHideTimer.Interval := fHideAfter;
         popup_form.autoHideTimer.Enabled := true;
      end else popup_form.autoHideTimer.Enabled := false;

      formVisible := true;

      wnd := getForegroundWindow;
      show;
      SetForegroundWindow(wnd);
   end;
end;

end.
