unit secMer_utils;

interface

uses sysUtils, ComCtrls, Forms, Classes, Dialogs, Controls, iniFiles;

type
   tSecMerUtil = class
   private
      fListView: tListView;
      count: smallInt;
   published
      constructor create;
      destructor free;

      procedure add;

      procedure onlyDel(const index: smallInt);
      procedure selectAfterDel(const index: smallInt);
      
      procedure delAll;
      procedure setAllActive(const value: boolean);
      procedure edit(const index: smallInt);
      procedure listCreate;
      procedure listSelect(const index: smallInt; const selected: boolean);

      procedure setActive(const index: smallInt; const value: boolean);
      procedure editItem(const index: smallInt);

      procedure timeMove;
      procedure reset(const index: smallInt);

      procedure showInfo(const index: smallInt);

      procedure saveList;
      procedure loadList;

      function areAnySelected: smallInt;

      property listView: tListView read fListView write fListView;

      function getFirstSelected: smallInt;
   end;
var
   secMer: tSecMerUtil;
implementation

uses global_vars, utils, windows;

{ tSecMer }

procedure{   }tSecMerUtil.add;
var
note: string;
active: boolean;
c: smallInt;
begin
   if not inputQuery(' ',
   '   ', note) then exit;

   if messageDlg('  ?', mtConfirmation, mbYesNo, 0) = mrYes then
   active := true else active := false;

   count := high(secMers);
   setLength(secMers, count+2);
   inc(Count);
   secMers[count].set_in := time+date;
   secMers[count].note := note;
   secMers[count].editing := false;

   secMers[count].l_sec := 0;
   secMers[count].l_min := 0;
   secMers[count].l_hour := 0;

   secMers[count].active := active;

   if areAnySelected <= 1 then
   begin
      secMers[count].selected := true;
      for c := 0 to count - 1 do
      secMers[c].selected := false;
   end;
   
   listCreate;
   if sl_select_index >= 0 then flistView.Items[count].Selected := true;
end;

function{     }tSecMerUtil.areAnySelected: smallInt;
var
c: smallInt;
begin
   result := 0;
   for c := 0 to high(secMers) do
      if secMers[c].selected then inc(result);
end;

constructor{  }tSecMerUtil.create;
begin
   fListView := tListView.Create(application);
end;

procedure{    }tSecMerUtil.delAll;
begin
   setLength(secMers, 0);
   fListView.Items.Clear;
   listSelect(-1, false);
   utils_form.del_all_secMers_button.enabled := false;
   utils_form._secMerDelall.Enabled := false;
   utils_form.s_allActive.Enabled := false;
   utils_form.s_allUnActive.Enabled := false;
   listCreate;
end;

procedure{  }tSecMerUtil.edit(const index: smallInt);
var
active: boolean;
begin
   secMers[index].editing := true;
   editItem(index);

   sl_select_index := index;

   secMers[index].note := inputBox(' ',
   '   ', secMers[index].note);

   {if messageDlg('  ?', mtInformation, mbYesNo, 0) = mrYes then
   active := true else active := false;}

   secMers[index].editing := false;

   //secMers[index].active := active;

   editItem(index);

   listSelect(index, index >= 0);
end;

procedure{   }tSecMerUtil.editItem(const index: smallInt);
var
tsl: tStringList;
ts: string;
begin
   tsl := tStringList.Create;
   ts := '';
   with fListView.Items.Item[index] do
   begin
      caption := intToStr(index + 1);
      tsl.Clear;
      tsl.Add(timeConverter(secMers[index].l_hour,
      secMers[index].l_min, secMers[index].l_sec));

      tsl.Add(secMers[index].note);
      tsl.Add(timeToStr(secMers[index].set_in)+' ['
         +dateToStr(secMers[index].set_in)+']');

      if not secMers[index].editing then
      begin
         if secMers[index].active then
         tsl.Add('')
         else
         tsl.Add('');
      end
      else tsl.Add('');

      tsl.Add(ts);
      subItems := tsl;
   end;
   tsl.free;
end;

destructor{   }tSecMerUtil.free;
begin
   fListView.Free;
end;

function{    }tSecMerUtil.getFirstSelected: smallInt;
var
c: smallInt;
begin
   result := -1;
   if high(secMers) < 0 then exit;

   for c := 0 to high(secMers) do
   begin
      if secMers[c].selected then
      begin
         result := c;
         exit;
      end;
   end;
end;

procedure{   }tSecMerUtil.listCreate;
var
c: smallInt;
begin
   fListView.Items.Clear;
   count := high(secMers);

   if count < 0 then
   begin
      utils_form.del_all_secMers_button.enabled := false;
      utils_form._secMerDelall.Enabled := false;
      utils_form.s_allActive.Enabled := false;
      utils_form.s_allUnActive.Enabled := false;
      utils_form.N30.Enabled := false;
      utils_form.N31.Enabled := false;
      utils_form.s_selectedActive.Enabled := false;
      utils_form.s_selectedUnActive.Enabled := false;
      utils_form.s_selectActive.Enabled := false;
      utils_form.s_selectUnActive.Enabled := false;
      utils_form.s_selectAll.Enabled := false;
      utils_form.s_selectInvert.Enabled := false;
      utils_form.N52.Enabled := false;
      exit;
   end;

   fListView.Items.BeginUpdate;
   for c := 0 to count do
   begin
      fListView.Items.Add;
      editItem(c);
   end;
   
   for c := 0 to high(secMers) do
   flistView.Items[c].Selected := secMers[c].selected;

   fListView.Items.EndUpdate;
   utils_form.del_all_secMers_button.enabled := true;
   utils_form._secMerDelall.Enabled := true;
   utils_form.s_allActive.Enabled := true;
   utils_form.s_allUnActive.Enabled := true;
   utils_form.N30.Enabled := true;
   utils_form.N31.Enabled := true;
   utils_form.s_selectActive.Enabled := true;
   utils_form.s_selectUnActive.Enabled := true;
   utils_form.s_selectAll.Enabled := true;
   utils_form.s_selectInvert.Enabled := true;
   utils_form.N52.Enabled := true;
end;

procedure{   }tSecMerUtil.listSelect(const index: smallInt; const selected: boolean);
var
anySelected: boolean;
selectedCount: smallInt;
begin
   if selected then sl_select_index := index else sl_select_index := -1;

   if index >= 0 then
   begin
      secMers[index].selected := selected;
      selectedCount := areAnySelected;
      anySelected := selectedCount > 0;
   end else
      anySelected := false;

   if anySelected then   
      statusBarCaption := ': '+
      intToStr(selectedCount)+'/'+intToStr(listView.Items.Count)
   else
      statusBarCaption := '';

   sl_first_selected := getFirstSelected;

   with utils_form do
   begin
      secMer_edit_button.enabled := anySelected;
      secMer_del_button.enabled := anySelected;
      secMer_active_button.enabled := anySelected;
      secMer_reset_button.enabled := anySelected;

      _secMeredit.Enabled := anySelected;
      _secMerdel.Enabled := anySelected;
      _secMeractive.Enabled := anySelected;
      _secMershowInfo.Enabled := selected;
      _secMerreset.Enabled := anySelected;

      s_selectedActive.enabled := anySelected;
      s_selectedUnActive.enabled := anySelected;

      if sl_first_selected >= 0 then
      begin
        secMer_active_button.down:= secMers[sl_first_selected].active;
        _secMeractive.Checked := secMers[sl_first_selected].active;
      end;
      status_bar.Panels[0].Text := statusBarCaption;
   end;
end;

procedure{   }tSecMerUtil.loadList;
var
f: tIniFile;
ts, f_way: string;
c, mh, ah, loadMode, changeStatus: smallInt;
active: boolean;
begin
   f_way := extractFilePath(application.ExeName)+'_work\secMers.ini';
   //if (not fileExists(f_way)) or (not utils_form.s17_btn.checked) then exit;
   f := tIniFile.Create(f_way);

   mh := f.ReadInteger('Count', 'Count', -1);
   if (mh = -1) or (not utils_form.s17_btn.checked) then
   begin
      f.Free;
      exit;
   end;
   //setLength(alarms, mh+1);

   loadMode := utils_form.s18_edit.ItemIndex;
   changeStatus := utils_form.s19_edit.ItemIndex;

   ah := -1;

   for c := 0 to mh do
   begin
      ts := 'SecMer'+intToStr(c);

      active := f.ReadBool(ts, 'active', false);

      if (active and (loadMode = 2)) or (not active and (loadMode = 1)) then
        continue;

      case changeStatus of
      1: if active then active := false;
      2: if not active then active := true;
      3: active := not active;
      end;
      //active := active <> changeStatus;

      inc(ah);
      setLength(secMers, ah+1);

      secMers[ah].active := active;
      secMers[ah].editing := false;
      secMers[ah].set_in := f.ReadFloat(ts, 'set_in', time+date);
      secMers[ah].note := f.ReadString(ts, 'note', '');
      secMers[ah].l_sec := f.ReadInteger(ts, 'l_sec', 0);
      secMers[ah].l_min := f.ReadInteger(ts, 'l_min', 0);
      secMers[ah].l_hour := f.ReadInteger(ts, 'l_hour', 0);
      secMers[ah].selected := false;
   end;
   f.Free;
end;

procedure tSecMerUtil.onlyDel(const index: smallInt);
var
counter: smallInt;
begin
   count := high(secMers);

   for counter := index to count - 1 do
   secMers[counter] := secMers[counter + 1];

   setLength(secMers, count);
end;

procedure{  }tSecMerUtil.reset(const index: smallInt);
begin
   secMers[index].l_hour := 0;
   secMers[index].l_min := 0;
   secMers[index].l_sec := 0;
   editItem(index);
end;

procedure{   }tSecMerUtil.saveList;
var
f: tIniFile;
f_way, ts: string;
c, mh: smallInt;
begin
   try
   f_way := extractFilePath(application.ExeName)+'_work\secMers.ini';

   if fileExists(f_way) then deleteFile(pChar(f_way));
   f := tIniFile.Create(f_way);

   mh := high(secMers);
   f.WriteInteger('Count', 'Count', mh);

   if mh = -1 then exit;
   for c := 0 to mh do
   begin
      ts := 'SecMer'+intToStr(c);
      f.writeBool(ts, 'active', secMers[c].active);
      f.WriteFloat(ts, 'set_in', secMers[c].set_in);
      f.WriteString(ts, 'note', secMers[c].note);
      f.WriteInteger(ts, 'l_sec', secMers[c].l_sec);
      f.WriteInteger(ts, 'l_min', secMers[c].l_min);
      f.WriteInteger(ts, 'l_hour', secMers[c].l_hour);
   end;
   f.Free;
   except

   end;
end;

procedure{   }tSecMerUtil.timeMove;
var
counter, h, m, s: smallInt;
begin
   count := high(secMers);
   if count < 0 then exit;
   fListView.Items.BeginUpdate;
   counter := 0;
   while counter <= count do
   begin
      if (not secMers[counter].active) or (secMers[counter].editing) then
      begin
         inc(counter);
         continue;
      end;
      h := secMers[counter].l_hour;
      m := secMers[counter].l_min;
      s := secMers[counter].l_sec;
      inc(s);
      if s = 60 then
      begin
         s := 0;
         inc(m);
         if m = 60 then
         begin
            m := 0;
            inc(h);
         end;
      end;
      secMers[counter].l_hour := h;
      secMers[counter].l_min := m;
      secMers[counter].l_sec := s;
      editItem(counter);
      inc(counter);
   end;
   fListView.Items.EndUpdate;
end;

procedure{    / }tSecMerUtil.setAllActive(const value: boolean);
var
counter: smallInt;
begin
   count := high(secMers);
   if count < 0 then exit;
   counter := 0;
   while counter <= count do
   begin
      secMers[counter].active := value;
      inc(counter);
   end;
   listCreate;

   //if sl_select_index >= 0 then flistView.Items[sl_select_index].Selected := true;
end;

procedure tSecMerUtil.selectAfterDel(const index: smallInt);
begin
   count := high(secMers);
   if count >= 0 then
      if index > count then sl_select_index := count else sl_select_index := index
   else
      sl_select_index := -1;

   listCreate;

   try
      flistView.SetFocus;
   except

   end;

   if sl_select_index >= 0 then
     flistView.Items[sl_select_index].Selected := true
   else
     listSelect(sl_select_index, sl_select_index >= 0);
end;

procedure{   }tSecMerUtil.setActive(const index: smallInt; const value: boolean);
begin
   secMers[index].active := value;
   editItem(index);
   {}utils_form._secMeractive.Checked := value;
   {}utils_form.secMer_active_button.Down := value;
end;

procedure{   }tSecMerUtil.showInfo(const index: smallInt);
var
str: string;
counter: smallInt;
begin
   str := '  '+secMer.listView.Items[index].Caption+';'; 
   str := str + #13#10+' (   ): ';
   str := str + timeConverter(secMers[index].l_hour, secMers[index].l_min,
      secMers[index].l_sec)+';';
   str := str + #13#10;
   str := str + ': "'+secMers[index].note+'";';
   str := str + #13#10;
   str := str + ' : '+TimeTostr(secMers[index].set_in)+
      ' ['+dateToStr(secMers[index].set_in)+'];';
   str := str + #13#10+': ';
   if secMers[index].active then
     str := str + ';' else str := str + ';';

   messageDlg(str, mtInformation, [mbOk], 0);
end;

end.
