unit timer_utils;

interface

uses
  SysUtils, ComCtrls, messages, global_vars, iniFiles;

type
   tTimerUtil = class
   private
      fListView: tListView;
      count: smallInt;
   published
      constructor create;
      destructor free;

      procedure add;

      procedure del(const index: smallInt);
      procedure onlyDel(const index: smallInt);
      procedure selectAfterDel(const index: smallInt);
      
      procedure delAll;
      procedure setAllActive(const value: boolean);
      procedure edit(const index: smallInt);
      procedure listCreate;
      procedure listSelect(const index: smallInt; const selected: boolean);

      procedure setActive(const index: smallInt; const value: boolean);
      procedure editItem(const index: smallInt);
      function atsFormShow(const edit: boolean): boolean;
      procedure assignValues(var index: timers_rec; const recLastEdited: boolean = true);
      procedure copyValues(const source: timers_rec; var dest: timers_rec);

      procedure saveList;
      procedure loadList;

      procedure timeMove;
      procedure reset(const index: smallInt);

      procedure timeOut(const index: smallInt);

      procedure showInfo(const index: smallInt);

      function areAnySelected: smallInt;

      property listView: tListView read fListView write fListView;

      function getFirstSelected: smallInt;

      procedure setValues(var index: timers_rec);
end;
var
   timer: tTimerUtil;
implementation

uses ats, utils, Dialogs, Controls, forms, Classes,
  soundPlay, Windows, dateUtils, popUp;

{ tTimer }

procedure{   }tTimerUtil.add;
var
c: smallInt;
begin
   if not atsFormShow(false) then exit;
   count := high(timers);
   setLength(timers, count+2);
   inc(Count);
   assignValues(timers[count]);

   if areAnySelected <= 1 then
   begin
      timers[count].selected := true;
      for c := 0 to count - 1 do
      timers[c].selected := false;
   end;

   listCreate;
end;

function{  -   }tTimerUtil.areAnySelected: smallInt;
var
c: smallInt;
begin
   result := 0;
   for c := 0 to high(timers) do
      if timers[c].selected then inc(result);
end;

procedure{   }tTimerUtil.assignValues(var index: timers_rec; const recLastEdited: boolean = true);
begin
  with at_addEdit_form do
  begin
    index.selected := false;
    index.time_out := false;
    index.editing := false;
    index.showPopUp := popUp_button.checked;
    index.del_timer := t_del_button.checked;
    index.set_unactive := t_active_button.checked;
    index.rings_unlimit := rings_unlimit_button.checked;
    index.reset_timer := t_reset_button.checked;
    index.ct_h := t_hc_input.Value;
    index.ct_m := t_mc_input.Value;
    index.ct_s := t_sc_input.Value;
    index.set_in := time+date;

    index.so_h := t_hs_input.Value;
    index.so_m := t_ms_input.Value;
    index.so_s := t_ss_input.Value;

    index.note := note_input.text;
    index.music_way := wav_File_input.Text;
    index.active := active_button.checked;
    index.use_wav := use_wav_button.checked and use_sound_button.Checked;
    index.say_time := say_time_button.checked and use_sound_button.Checked;
    index.rings_num := ring_nums_edit.Value;
  end;
  if recLastEdited then
  begin
     copyValues(index, le_Timer.rec);
     le_Timer.exists := true;
  end;
end;

function{ ,  }tTimerUtil.atsFormShow(const edit: boolean): boolean;
var
allRight, error: boolean;
h, m, s, mr: smallInt;
//timeChanged: boolean;
messageText: string;
begin
   allRight := false;
   with at_addEdit_form do
   begin
      if isWindowVisible(handle) then
      begin
         messageDlg(' /  !'+#13#10+
         '      !', mtWarning, [mbOk], 0);
         exit;
      end;

      clientHeight := 280;
      t_changeSynch_button.Checked := not edit;
      if not edit then
      begin
         caption := ' ';
         icon.LoadFromResourceName(hInstance, 'add');

         if utils_form.s1_btn.checked and le_timer.exists then
         begin
            setValues(le_timer.rec);
         end
         else
         begin
            t_hs_input.Value := 0;
            t_ms_input.Value := 0;
            t_ss_input.Value := 0;

            t_hc_input.Value := 0;
            t_mc_input.Value := 0;
            t_sc_input.Value := 0;

            use_sound_button.Checked := true;
            use_wav_button.Checked := true;
            use_sound_button.OnClick(nil);

            if wav_file_input.Items.Count > 0 then
              wav_file_input.Text := wav_file_input.Items.Strings[0]
              else wav_file_input.Text := '';
            note_input.Text := '';
            ring_nums_edit.Value := 1;
            ring_nums_edit.Enabled := true;
            active_button.checked := true;
            popUp_button.checked := true;
            rings_unlimit_button.checked := false;
            t_del_button.checked := true;
            t_reset_button.checked := false;
            t_active_button.checked := false;
         end;
      end
      else
      begin
         caption := ' ';
         icon.LoadFromResourceName(hInstance, 'edit');

         setValues(timers[tl_select_index]);

         {s := t_s_input.Value;
         m := t_m_input.Value;
         h := t_h_input.Value;}

         rings_unlimit_buttonClick(nil);
         use_sound_button.OnClick(nil);
      end;
      alarm_panel.visible := false;
      timer_panel.visible := true;
   end; 
   repeat
      isOk := false;
      messageText := '';
      error := false;
      with at_addEdit_form do
      begin
         showModal;
         if not isOk then exit;
         try
            {timeChanged := (h <> t_h_input.Value) or
                           (m <> t_m_input.Value) or
                           (s <> t_s_input.Value); }

            h := t_hs_input.value;
            m := t_ms_input.value;
            s := t_ss_input.value;

            if (h = 0) and (m = 0) and (s = 0){ and (not edit)} then
            begin
               mr := messageDlg('   ,   1 .'+#13#10+
               '   1 ?', mtWarning, mbYesNoCancel, 0);
               if mr = mrYes then
               begin
                  t_hs_input.value := 0;
                  t_ms_input.value := 0;
                  t_ss_input.value := 1;
                  h := 0; m := 0; s := 1;
               end;
               if mr = mrNo then
               begin
                  allRight := false;
                  continue;
               end;
               if mr = mrCancel then
               begin
                  result := false;
                  exit;
               end;
            end;

            if edit then
            begin
               {mr := messageDlg('     ?'+#13#10+
               ' -   ;', mtWarning, mbYesNoCancel, 0);
               if mr = mrYes then
               begin
                  changeStart := true;
               end;
               if mr = mrNo then
               begin
                  changeStart := false;
                  //continue;
               end;  }
               if mr = mrCancel then
               begin
                  result := false;
                  exit;
               end;
            end;

            if (h > 168) or (m > 59) or (s > 59) then
            begin
               error := true;
               messageText := '   !'+#13#10;
            end;
         except
            error := true;
            messageText := '   !'+#13#10;
         end;

         if use_wav_button.checked and use_sound_button.Checked and not fileExists(wav_file_input.Text) then
         begin
            error := true;
            messageText := messageText+'    !'+#13#10;
         end;
      end;
      if error then
      begin
         mr := messageDlg(messageText+'?', mtWarning, mbYesNoCancel, 0);
         if mr = mrYes then
            allRight := true;
         if mr = mrNo then
            allRight := false;
         if mr = mrCancel then
         begin
            result := false;
            exit;
         end;
      end else allRight := true;
   until allRight;
   result := isOk;
end;


procedure{   }tTimerUtil.selectAfterDel(const index: smallInt);
begin
   count := high(timers);

   if count >= 0 then
      if index > count then
         tl_select_index := count
      else
         tl_select_index := index
   else
      tl_select_index := -1;

   listCreate;

   try
      flistView.SetFocus;
   except

   end;

   if tl_select_index >= 0 then
     flistView.Items[tl_select_index].Selected := true
   else
     listSelect(tl_select_index, tl_select_index >= 0);
end;

procedure tTimerUtil.copyValues(const source: timers_rec; var dest: timers_rec);
begin
   dest.time_out := source.time_out;
   dest.showPopUp := source.showPopUp;
   dest.del_timer := source.del_timer;
   dest.set_unactive := source.set_unactive;
   dest.rings_unlimit := source.rings_unlimit;
   dest.reset_timer := source.reset_timer;
   dest.ct_h := source.ct_h;
   dest.ct_m := source.ct_m;
   dest.ct_s := source.ct_s;

   dest.so_h := source.so_h;
   dest.so_m := source.so_m;
   dest.so_s := source.so_s;
   //dest.set_in := source.set_in;
   dest.note := source.note;
   dest.music_way := source.music_way;
   dest.active := source.active;
   dest.use_wav := source.use_wav;
   dest.say_time := source.say_time;
   dest.rings_num := source.rings_num;
end;

constructor{  }tTimerUtil.create;
begin
   fListView := tListView.create(application);
end;

procedure{   }tTimerUtil.del(const index: smallInt);
begin
   onlyDel(index);
   selectAfterDel(index);
end;

procedure{    }tTimerUtil.delAll;
begin
   setLength(timers, 0);
   fListView.Items.Clear;
   listSelect(-1, false);
   utils_form.del_all_timers_button.enabled := false;
   //utils_form.t_del_all.Enabled := false;
   utils_form.t_allActive.Enabled := false;
   utils_form.t_allUnActive.Enabled := false;
   listCreate;
end;

procedure{  }tTimerUtil.edit(const index: smallInt);
var
changeStart: boolean;
begin
   timers[index].editing := true;
   editItem(index);
   tl_select_index := index;
   if not atsFormShow(true) then
   begin
      timers[index].editing := false;
      editItem(index);
      exit;
   end;
   timers[index].editing := false;
   assignValues(timers[index]);
   editItem(index);
   listSelect(index, index >= 0);
end;

procedure{   }tTimerUtil.editItem(const index: smallInt);
var
tsl: tStringList;
ts: string;
begin
   tsl := tStringList.Create;
   ts := '';
   with fListView.Items.Item[index] do
   begin
      caption := intToStr(index+1);
      tsl.Clear;
      tsl.Add(timeToStr(encodeTime(timers[index].so_h, timers[index].so_m, timers[index].so_s, 0)));
      tsl.Add(timers[index].note);
      tsl.Add(timeToStr(timers[index].set_in)+' ['
         +dateToStr(timers[index].set_in)+']');
      tsl.Add(timeConverter(timers[index].ct_h,
      timers[index].ct_m, timers[index].ct_s));

      if not timers[index].editing then
      begin
         if timers[index].active then
         tsl.Add('')
         else
         tsl.Add('');
      end
      else tsl.Add('');

      if timers[index].use_wav then
      begin
         if timers[index].rings_unlimit then
         ts := '  "'+timers[index].music_way+'"; '
         else
         ts := ' "'+timers[index].music_way+'" '+intToStr(timers[index].rings_num)+
         ' .; ';
      end;

      if timers[index].say_time then
         ts := ' ; ';

      if timers[index].showPopUp then
         ts := ts + '  ; ';
      
      if timers[index].del_timer then
      ts := ts + ' ; ';
      if timers[index].reset_timer then
      begin
         if timers[index].set_unactive then
         ts := ts + '     ; '
         else ts := ts + ' ; ';
      end;

      tsl.Add(ts);
      subItems := tsl;
   end;
   tsl.free;
end;

destructor{   }tTimerUtil.free;
begin
   listView.Free;
end;

function{    }tTimerUtil.getFirstSelected: smallInt;
var
c: smallInt;
begin
   result := -1;
   if high(timers) < 0 then exit;

   for c := 0 to high(timers) do
   begin
      if timers[c].selected then
      begin
         result := c;
         exit;
      end;
   end;
end;

procedure{   }tTimerUtil.listCreate;
var
c: smallInt;
begin
   fListView.Items.Clear;
   count := high(timers);

   if count < 0 then
   begin
      utils_form.del_all_timers_button.enabled := false;
      utils_form._timerDelall.Enabled := false;
      utils_form.t_allActive.Enabled := false;
      utils_form.t_allUnActive.Enabled := false;
      utils_form.N34.Enabled := false;
      utils_form.N36.Enabled := false;
      utils_form.t_selectedActive.Enabled := false;
      utils_form.t_selectedUnActive.Enabled := false;
      utils_form.t_selectActive.Enabled := false;
      utils_form.t_selectUnActive.Enabled := false;
      utils_form.t_selectAll.Enabled := false;
      utils_form.t_selectInvert.Enabled := false;
      utils_form.N47.Enabled := false;
      exit;
   end;

   fListView.Items.BeginUpdate;
   for c := 0 to count do
   begin
      fListView.Items.Add;
      editItem(c);
   end;
   
   for c := 0 to high(timers) do
   flistView.Items[c].Selected := timers[c].selected;

   fListView.Items.EndUpdate;
   utils_form.del_all_timers_button.enabled := true;
   utils_form._timerDelall.Enabled := true;
   utils_form.t_allActive.Enabled := true;
   utils_form.t_allUnActive.Enabled := true;
   utils_form.N34.Enabled := true;
   utils_form.N36.Enabled := true;
   utils_form.t_selectActive.Enabled := true;
   utils_form.t_selectUnActive.Enabled := true;
   utils_form.t_selectAll.Enabled := true;
   utils_form.t_selectInvert.Enabled := true;
   utils_form.N47.Enabled := true;
end;

procedure{   }tTimerUtil.listSelect(const index: smallInt; const selected: boolean);
var
anySelected: boolean;
selectedCount: smallInt;
begin
   if selected then tl_select_index := index else tl_select_index := -1;

   if index >= 0 then
   begin
      timers[index].selected := selected;
      selectedCount := areAnySelected;
      anySelected := selectedCount > 0;
   end else
      anySelected := false;

   if anySelected then   
      statusBarCaption := ': '+
      intToStr(selectedCount)+'/'+intToStr(listView.Items.Count)
   else
      statusBarCaption := '';

   tl_first_selected := getFirstSelected;

   with utils_form do
   begin
      timer_edit_button.enabled := anySelected;
      timer_del_button.enabled := anySelected;
      timer_active_button.enabled := anySelected;
      timer_reset_button.enabled := anySelected;

      _timeredit.Enabled := anySelected;
      _timerdel.Enabled := anySelected;
      _timeractive.Enabled := anySelected;
      _timershowInfo.Enabled := selected;
      _timerreset.Enabled := anySelected;

      t_selectedActive.enabled := anySelected;
      t_selectedUnActive.enabled := anySelected;

      if tl_first_selected >= 0 then
      begin
        timer_active_button.down:= timers[tl_first_selected].active;
        _timeractive.Checked := timers[tl_first_selected].active;
      end;
      status_bar.Panels[0].Text := statusBarCaption;
   end;
end;

procedure{   }tTimerUtil.loadList;
label finished, profiles;
var
f: tIniFile;
ts, f_way: string;
c, mh, ah, loadMode, changeStatus: smallInt;
active: boolean;
begin
   f_way := extractFilePath(application.ExeName)+'_work\timers.ini';
   //if (not fileExists(f_way)) or (not utils_form.s14_btn.checked) then exit;
   f := tIniFile.Create(f_way);

   mh := f.ReadInteger('Count', 'Count', -1);
   if (mh = -1) or (not utils_form.s14_btn.checked) then goto profiles;
   //setLength(timers, mh+1);

   loadMode := utils_form.s15_edit.ItemIndex;
   changeStatus := utils_form.s16_edit.ItemIndex;

   ah := -1;

   for c := 0 to mh do
   begin
      ts := 'Timer'+intToStr(c);

      active := f.ReadBool(ts, 'active', false);

      if (active and (loadMode = 2)) or (not active and (loadMode = 1)) then
        continue;

      case changeStatus of
      1: if active then active := false;
      2: if not active then active := true;
      3: active := not active;
      end;

      inc(ah);
      setLength(timers, ah+1);

      timers[ah].active := active;
      timers[ah].editing := false;
      timers[ah].showPopUp := f.readBool(ts, 'popUp', true);
      timers[ah].set_unactive := f.ReadBool(ts, 'set_unactive', false);
      timers[ah].rings_unlimit := f.ReadBool(ts, 'rings_unlimit', false);
      timers[ah].rings_num := f.ReadInteger(ts, 'rings_num', 1);
      timers[ah].del_timer := f.ReadBool(ts, 'del_timer', true);
      timers[ah].reset_timer := f.ReadBool(ts, 'reset_timer', false);
      timers[ah].time_out := f.ReadBool(ts, 'time_out', false);
      timers[ah].set_in := f.ReadFloat(ts, 'set_in', time+date);
      //timers[ah].set_on := f.ReadFloat(ts, 'set_on', encodeTime(0, 1, 0, 0));
      timers[ah].note := f.ReadString(ts, 'note', '');
      timers[ah].music_way := f.ReadString(ts, 'music_way', at_addedit_form.wav_file_input.Text);
      timers[ah].use_wav := f.ReadBool(ts, 'use_wav', false);
      timers[ah].say_time := f.ReadBool(ts, 'say_time', false);
      timers[ah].ct_h := f.ReadInteger(ts, 'ct_h', 0);
      timers[ah].ct_m := f.ReadInteger(ts, 'ct_m', 0);
      timers[ah].ct_s := f.ReadInteger(ts, 'ct_s', 0);

      timers[ah].so_h := f.ReadInteger(ts, 'so_h', 0);
      timers[ah].so_m := f.ReadInteger(ts, 'so_m', 0);
      timers[ah].so_s := f.ReadInteger(ts, 'so_s', 0);
      timers[ah].selected := false;
   end;

   profiles:
   mh := f.ReadInteger('Count', 'profilesCount', -1);
   if (mh = -1) then goto finished;
   ah := -1;
   for c := 0 to mh do
   begin
      ts := 'Profile'+intToStr(c);

      inc(ah);
      setLength(timerProfiles, ah+1);
      timerProfiles[ah].name := f.readString(ts, 'name', '( )');
      timerProfiles[ah].rec.active := f.ReadBool(ts, 'active', false);
      timerProfiles[ah].rec.showPopUp := f.readBool(ts, 'popUp', true);
      timerProfiles[ah].rec.set_unactive := f.ReadBool(ts, 'set_unactive', false);
      timerProfiles[ah].rec.rings_unlimit := f.ReadBool(ts, 'rings_unlimit', false);
      timerProfiles[ah].rec.rings_num := f.ReadInteger(ts, 'rings_num', 1);
      timerProfiles[ah].rec.del_timer := f.ReadBool(ts, 'del_timer', true);
      timerProfiles[ah].rec.reset_timer := f.ReadBool(ts, 'reset_timer', false);
      timerProfiles[ah].rec.time_out := f.ReadBool(ts, 'time_out', false);
      timerProfiles[ah].rec.note := f.ReadString(ts, 'note', '');
      timerProfiles[ah].rec.music_way := f.ReadString(ts, 'music_way', at_addedit_form.wav_file_input.Text);
      timerProfiles[ah].rec.use_wav := f.ReadBool(ts, 'use_wav', false);
      timerProfiles[ah].rec.say_time := f.ReadBool(ts, 'say_time', false);
      timerProfiles[ah].rec.ct_h := f.ReadInteger(ts, 'ct_h', 0);
      timerProfiles[ah].rec.ct_m := f.ReadInteger(ts, 'ct_m', 0);
      timerProfiles[ah].rec.ct_s := f.ReadInteger(ts, 'ct_s', 0);

      timerProfiles[ah].rec.so_h := f.ReadInteger(ts, 'so_h', 0);
      timerProfiles[ah].rec.so_m := f.ReadInteger(ts, 'so_m', 0);
      timerProfiles[ah].rec.so_s := f.ReadInteger(ts, 'so_s', 0);
   end;


   finished: f.Free;

   le_timer.exists := false;
end;

procedure tTimerUtil.onlyDel(const index: smallInt);
var
counter: smallInt;
begin
   count := high(timers);

   for counter := index to count - 1 do
   timers[counter] := timers[counter + 1];

   setLength(timers, count);
end;

procedure{  }tTimerUtil.reset(const index: smallInt);
begin
   timers[index].ct_h := timers[index].so_h;
   timers[index].ct_m := timers[index].so_m;
   timers[index].ct_s := timers[index].so_s;
   editItem(index);
end;

procedure{   }tTimerUtil.saveList;
label profiles, finished;
var
f: tIniFile;
f_way, ts: string;
c, mh: smallInt;
begin
   try
   f_way := extractFilePath(application.ExeName)+'_work\timers.ini';

   if fileExists(f_way) then deleteFile(pChar(f_way));
   f := tIniFile.Create(f_way);

   mh := high(timers);
   f.WriteInteger('Count', 'Count', mh);

   if mh = -1 then
      goto profiles;

   for c := 0 to mh do
   begin
      ts := 'Timer'+intToStr(c);
      f.writeBool(ts, 'active', timers[c].active);
      f.WriteBool(ts, 'popUp', timers[c].showPopUp);
      f.writeBool(ts, 'set_unactive', timers[c].set_unactive);
      f.writeBool(ts, 'rings_unlimit', timers[c].rings_unlimit);
      f.writeInteger(ts, 'rings_num', timers[c].rings_num);
      f.writeBool(ts, 'del_timer', timers[c].del_timer);
      f.writeBool(ts, 'reset_timer', timers[c].reset_timer);
      f.WriteFloat(ts, 'set_in', timers[c].set_in);
      f.WriteString(ts, 'note', timers[c].note);
      f.WriteString(ts, 'music_way', timers[c].music_way);
      f.WriteBool(ts, 'use_wav', timers[c].use_wav);
      f.WriteBool(ts, 'say_time', timers[c].say_time);
      f.WriteBool(ts, 'time_out', timers[c].time_out);
      f.WriteInteger(ts, 'ct_h', timers[c].ct_h);
      f.WriteInteger(ts, 'ct_m', timers[c].ct_m);
      f.WriteInteger(ts, 'ct_s', timers[c].ct_s);

      f.WriteInteger(ts, 'so_h', timers[c].so_h);
      f.WriteInteger(ts, 'so_m', timers[c].so_m);
      f.WriteInteger(ts, 'so_s', timers[c].so_s);
   end;

   profiles:
   mh := high(timerProfiles);
   f.WriteInteger('Count', 'profilesCount', mh);

   if mh = -1 then
      goto finished;

   for c := 0 to mh do
   begin
      ts := 'Profile'+intToStr(c);
      f.writeString(ts, 'name', timerProfiles[c].name);
      f.writeBool(ts, 'active', timerProfiles[c].rec.active);
      f.WriteBool(ts, 'popUp', timerProfiles[c].rec.showPopUp);
      f.writeBool(ts, 'set_unactive', timerProfiles[c].rec.set_unactive);
      f.writeBool(ts, 'rings_unlimit', timerProfiles[c].rec.rings_unlimit);
      f.writeInteger(ts, 'rings_num', timerProfiles[c].rec.rings_num);
      f.writeBool(ts, 'del_timer', timerProfiles[c].rec.del_timer);
      f.writeBool(ts, 'reset_timer', timerProfiles[c].rec.reset_timer);
      f.WriteString(ts, 'music_way', timerProfiles[c].rec.music_way);
      f.WriteBool(ts, 'use_wav', timerProfiles[c].rec.use_wav);
      f.WriteBool(ts, 'say_time', timerProfiles[c].rec.say_time);
      f.WriteBool(ts, 'time_out', timerProfiles[c].rec.time_out);
      f.writeString(ts, 'note', timerProfiles[c].rec.note);
      f.WriteInteger(ts, 'ct_h', timerProfiles[c].rec.ct_h);
      f.WriteInteger(ts, 'ct_m', timerProfiles[c].rec.ct_m);
      f.WriteInteger(ts, 'ct_s', timerProfiles[c].rec.ct_s);

      f.WriteInteger(ts, 'so_h', timerProfiles[c].rec.so_h);
      f.WriteInteger(ts, 'so_m', timerProfiles[c].rec.so_m);
      f.WriteInteger(ts, 'so_s', timerProfiles[c].rec.so_s);
   end;      

   finished: f.Free;
   except

   end;
end;

procedure{    / }tTimerUtil.setAllActive(const value: boolean);
var
counter: smallInt;
begin
   count := high(timers);
   if count < 0 then exit;
   counter := 0;
   while counter <= count do
   begin
      timers[counter].active := value;
      inc(counter);
   end;
   listCreate;

   //if tl_select_index >= 0 then flistView.Items[tl_select_index].Selected := true;
end;

procedure tTimerUtil.setValues(var index: timers_rec);
begin
   with at_addedit_form do
   begin
      t_hc_input.Value := index.ct_h;
      t_mc_input.Value := index.ct_m;
      t_sc_input.Value := index.ct_s;

      t_hs_input.Value := index.so_h;
      t_ms_input.Value := index.so_m;
      t_ss_input.Value := index.so_s;

      //utils_form.caption := intToStr(index);

      use_sound_button.checked := index.use_wav or
         index.say_time;
      use_wav_button.Checked := index.use_wav;
      say_time_button.Checked := index.say_time;

      wav_file_input.Text := index.music_way;
      note_input.Text := index.note;
      ring_nums_edit.Value := index.rings_num;
      active_button.checked := index.active;
      popUp_button.checked := index.showPopUp;
      rings_unlimit_button.checked := index.rings_unlimit;
      t_del_button.checked := index.del_timer;
      t_reset_button.checked := index.reset_timer;
      t_active_button.checked := index.set_unactive;
   end;
end;

procedure{   }tTimerUtil.setActive(const index: smallInt; const value: boolean);
begin
   timers[index].active := value;
   editItem(index);
   {}utils_form._timeractive.Checked := value;
   {}utils_form.timer_active_button.Down := value;
end;

procedure{   }tTimerUtil.showInfo(const index: smallInt);
var
str: string;
begin
   str := '  '+timer.listView.Items[index].Caption+';';
   str := str+#13#10+' : '+ timeToStr(encodeTime(timers[index].so_h, timers[index].so_m, timers[index].so_s, 0))+';';
   str := str + #13#10+' (   ): ';
   str := str + timeConverter(timers[index].ct_h, timers[index].ct_m,
      timers[index].ct_s)+';';
   str := str + #13#10;
   str := str + ': "'+timers[index].note+'";';
   str := str + #13#10;
   str := str + ' : '+TimeToStr(timers[index].set_in)+
      ' ['+dateToStr(timers[index].set_in)+'];';
   str := str + #13#10+': ';
   if timers[index].active then
     str := str + ';' else str := str + ';';
   str := str + #13#10+': ';
   if timers[index].use_wav then
   begin
     if timers[index].rings_unlimit then
     str := str + '  "'+timers[index].music_way+'"; '
     else
     str := str + ' "'+timers[index].music_way+'" '+intToStr(timers[index].rings_num)+
     ' .; ';
   end;
   if timers[index].say_time then
     str := str + ' ; ';
   if timers[index].showPopUp then
     str := str + #13#10+ '  ; ';
   str := str + #13#10;
   if timers[index].del_timer then
      str := str + ' ;';
   if timers[index].reset_timer then
   begin
      if timers[index].set_unactive then
      str := str + '     ;'
      else str := str + ' ;';
   end;

   messageDlg(str, mtInformation, [mbOk], 0);
end;

procedure{   }tTimerUtil.timeMove;
var
counter, h, m, s: smallInt;
del: boolean;
haveToDel: array of smallInt;
begin
   count := high(timers);
   if count < 0 then exit;
   fListView.Items.BeginUpdate;
   counter := 0;
   t_haveToPlay := false;
   t_haveToRepaint := false;
   while counter <= count do
   begin
      if counter > count then break;

      if (not timers[counter].active) or (timers[counter].editing) then
      begin
         inc(counter);
         continue;
      end;
      h := timers[counter].ct_h;
      m := timers[counter].ct_m;
      s := timers[counter].ct_s;

      dec(s);
      if s < 0 then
      begin
         s := 59;
         dec(m);
         if m < 0 then
         begin
            m := 59;
            dec(h);
            if h < 0 then
            begin
               s := 0;
               m := 0;
               h := 0;
               //timers[counter].time_out := true;
            end;
         end;
      end;

      {if s > 0 then
         dec(s)
      else
      begin
         if m > 0 then
         begin
            s := 59;
            dec(m);
         end
         else
         begin
            if h > 0 then
            begin
               s := 59;
               m := 59;
               dec(h);
            end
            else
            begin
               s := 0;
               m := 0;
               h := 0;
            end;
         end;
      end;}

      if (s = 0) and (m = 0) and (h = 0) then
         timers[counter].time_out := true;

      if timers[counter].time_out then
      begin
         if timers[counter].del_timer then
         begin
            setLength(haveToDel, high(haveToDel)+2);
            haveToDel[high(haveToDel)] := counter;
         end;
         //else t_haveToRepaint := true;

         timeOut(counter);
         {if not del then
         begin
            editItem(counter);
            if tl_select_index >= 0 then listSelect(tl_select_index, true);
            inc(counter);
         end;
         continue; }
      end
      else
      begin
         timers[counter].ct_h := h;
         timers[counter].ct_m := m;
         timers[counter].ct_s := s;
         editItem(counter);
      end;
      inc(counter);
   end;

   if high(haveToDel) >= 0 then
   begin
      for counter := high(haveToDel) downto 0 do
         timer.onlyDel(haveToDel[counter]);

      //if areAnySelected = 0 then
         listCreate
      {else
         alarm.selectAfterDel(lastDeleted);}
   end;

   if t_haveToPlay then _player.play;
   fListView.Items.EndUpdate;
end;

procedure{   }tTimerUtil.timeOut(const index: smallInt);
var
caption: string;
begin
   setRinging(true);

   caption := timeToStr(time) + ' ['+DateToStr(date)+'] : '+timers[index].note;

   if timers[index].showPopUp then
   begin
      nowRingingPage := 1;
      showPopUpWnd(' !!!', caption);
   end;

   if noteHistory = '' then
   noteHistory := caption
   else
   noteHistory := caption+#13#10+noteHistory;

   if timers[index].use_wav then
   begin
      if fileExists(timers[index].music_way) then
      begin
         _player.stop;
         _player.way := timers[index].music_way;
         _player.repeatNums := timers[index].rings_num;
         _player.RepeatUnlimit := timers[index].rings_unlimit;
         //_player.play;
         t_haveToPlay := true;
      end;
   end;

   if timers[index].say_time and (@sayTime <> nil) and AllowPlaying then
   begin
      AllowPlaying(true);
      sayTime(hourOf(time), minuteOf(time), messageSendHandle, WM_USER+1);
   end;

   if timers[index].reset_timer then
   begin
      reset(index);
      timers[index].time_out := false;
   end;
   if timers[index].set_unactive then timers[index].active := false;
   //if timers[index].del_timer then del(index);
end;

end.
