unit TimeDateLabel;

interface

uses
  SysUtils, Classes, Controls, StdCtrls, ExtCtrls, Forms, Windows,
  Messages, Graphics, dialogs;
type
  tShowTypes = (stTime, stDate, stTimeDate, stDateTime);
  TTimeDateLabel = class(TLabel)
  private
    { Private declarations }
    timer: tTimer;
    fNowTime: string[8];
    fNowDate: string[10];
    fOnRefresh: tNotifyEvent;
    fRefreshInterval: cardinal;
    fday, fMonth, fHour, fMinute, fSecond, fYear: word;
    fShowTypes: tShowTypes;
    fIs12, fIsPm: boolean;
    fActive: boolean;
    fTimeDate: tDateTime;
    fTimeSep, fDateSep, fTimeDateSep: string;
    procedure setDateSep(const Value: string);
    procedure setTimeDateSep(const Value: string);
    procedure setTimeSep(const Value: string);
    function getTimeDate: tDateTime;
    procedure setTimeDate(const Value: tDateTime);
    procedure setShowTypes(const Value: tShowTypes);
    procedure setRefreshInterval(const Value: cardinal);
    procedure setActive(const Value: boolean);
    procedure setIs12(const Value: boolean);
  protected
    { Protected declarations }
    procedure onTimer(sender: tObject);
    procedure showValues;
  public
    { Public declarations }
  published
    { Published declarations }
    constructor create(aOwner: tComponent); override;
    destructor free;

    property showTypes: tShowTypes read fShowTypes write setShowTypes;
    property refreshInterval: cardinal read fRefreshInterval write setRefreshInterval;
    property onRefresh: tNotifyEvent read fOnRefresh write fOnRefresh;

    property day: word read fDay;
    property month: word read fMonth;
    property year: word read fYear;

    property seconds: word read fSecond;
    property minutes: word read fMinute;
    property hours: word read fHour;

    property is12: boolean read fIs12 write setIs12;
    property isPm: boolean read fIsPm;
    property active: boolean read fActive write setActive;

    property timeDate: tDateTime read getTimeDate write setTimeDate;

    property timeSep: string read fTimeSep write setTimeSep;
    property dateSep: string read fDateSep write setDateSep;
    property timeDateSep: string read fTimeDateSep write setTimeDateSep;

    procedure getTime(var hours, minutes, seconds: word);
    procedure getDate(var days, months, years: word);
    function timeConverter(const h, m, s: word): string;
    function dateConverter(const d, m: byte; y: word): string;
end;

procedure Register;

implementation

procedure{  }Register;
begin
   RegisterComponents('atomAltera', [TTimeDateLabel]);
end;

{ TTimeDateLabel }

constructor{  }TTimeDateLabel.create(aOwner: tComponent);
begin
   inherited create(aOwner);
   timer := tTimer.create(self);
   timer.interval := 1000;
   timer.enabled := true;
   timer.onTimer := onTimer;
   fActive := true;
   fRefreshInterval := 1000;
   fTimeSep := ':';
   fDateSep := '.';
   fTimeDateSep := '_';
end;

function{   }TTimeDateLabel.dateConverter(const d, m: byte; y: word): string;
var
day, month, year: string;
begin
   day := intToStr(d);
   month := intToStr(m);
   year := intToStr(y);

   if length(day) < 2 then day := '0' + day;
   if length(month) < 2 then month := '0' + month;
   result := day+fDateSep+month+fDateSep+year;
end;

destructor{  }TTimeDateLabel.free;
begin
   timer.Free;
   inherited free;
end;

procedure{   }TTimeDateLabel.getDate(var days, months, years: word);
var
dt: tSystemTime;
begin
   getLocalTime(dt);

   days := dt.wDay;
   months := dt.wMonth;
   years := dt.wYear;
end;

procedure{   }TTimeDateLabel.getTime(var hours, minutes, seconds: word);
var
dt: tSystemTime;
begin
   getLocalTime(dt);

   seconds := dt.wSecond;
   minutes := dt.wMinute;
   hours := dt.wHour;

   if fIs12 then
   begin
      if hours >=12 then
      begin
         if hours > 12 then hours := hours - 12;
         fIsPm := true;
      end 
      else 
      begin
         if hours = 0 then hours := 12;
         fIsPm := false;    
      end;       
   end;

   {if fIs12 then
   begin
      if hours >=12 then
      begin
         if hours > 12 then hours := hours - 12;
         fIsPm := true;
      end else fIsPm := false;
   end;}
end;

function TTimeDateLabel.getTimeDate: tDateTime;
begin
   result := enCodeTime(fHour, fMinute, fSecond, 0)+
      enCodeDate(fYear, fMonth, fDay);
end;

procedure{   }TTimeDateLabel.onTimer(sender: tObject);
begin
   showValues;
end;

procedure TTimeDateLabel.setActive(const Value: boolean);
begin
   fActive := Value;
   timer.Enabled := value;
end;

procedure TTimeDateLabel.setDateSep(const Value: string);
begin
   fDateSep := Value;
   showValues
end;

procedure TTimeDateLabel.setIs12(const Value: boolean);
begin
   fIs12 := Value;
   showValues;
end;

procedure TTimeDateLabel.setRefreshInterval(const Value: cardinal);
begin
   fRefreshInterval := Value;
   timer.Interval := value;
end;

procedure TTimeDateLabel.setShowTypes(const Value: tShowTypes);
begin
   fShowTypes := Value;
   showValues;
end;

procedure TTimeDateLabel.setTimeDate(const Value: tDateTime);
var
th, tm, ts, tms, dy, dm, dd: word;
begin
   decodeTime(value, th, tm, ts, tms);
   decodeDate(value, dy, dm, dd);

   fHour := th;
   fMinute := tm;
   fSecond := ts;

   fYear := dy;
   fMonth := dm;
   fDay := dd;

   showValues;
end;

procedure TTimeDateLabel.setTimeDateSep(const Value: string);
begin
   fTimeDateSep := Value;
   showValues
end;

procedure TTimeDateLabel.setTimeSep(const Value: string);
begin
   fTimeSep := Value;
   showValues
end;

procedure{   }TTimeDateLabel.showValues;
var
caption_str: string[19];
begin
   getTime(fHour, fMinute, fSecond);
   getDate(fDay, fMonth, fYear);

   if fShowTypes = stTime then
   caption_str := timeConverter(fHour, fMinute, fSecond);

   if fShowTypes = stDate then
   caption_str := dateConverter(fDay, fMonth, fYear);

   if fShowTypes = stTimeDate then
   caption_str := timeConverter(fHour, fMinute, fSecond)+fTimeDateSep+dateConverter(fDay, fMonth, fYear);

   if fShowTypes = stDateTime then
   caption_str := dateConverter(fDay, fMonth, fYear)+fTimeDateSep+timeConverter(fHour, fMinute, fSecond);

   caption := caption_str;

   if assigned(fOnRefresh) then fOnRefresh(self);
end;

function{   }TTimeDateLabel.timeConverter(const h, m, s: word): string;
var
second, minute, hour: string;
begin
   second := intToStr(s);
   minute := intToStr(m);
   hour := intToStr(h);

   if length(second) < 2 then
   begin
      if length(second) < 1 then second := '00'
      else second := '0' + second;
   end;
   if length(minute) < 2 then
   begin
      if length(minute) < 1 then minute := '00'
      else minute := '0' + minute;
   end;
   if length(hour) < 2 then
   begin
      if length(hour) < 1 then hour := '00'
      else hour := '0' + hour;
   end;
   result := hour+fTimeSep+minute+fTimeSep+second;
end;

end.
